@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2013 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2013-2016 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 #include "command.h"
 
 // PKG_ADD: autoload ("pq_update_types", "pq_interface.oct");
+// PKG_DEL: autoload ("pq_update_types", "pq_interface.oct", "remove");
 
 DEFUN_DLD (pq_update_types, args, ,
            "-*- texinfo -*-\n\
@@ -47,7 +48,8 @@ Updates information on existing postgresql types for @var{connection}. Use this
   const octave_pq_connection &oct_pq_conn =
     dynamic_cast<const octave_pq_connection&> (rep);
 
-  oct_pq_conn.get_rep ()->octave_pq_refresh_types ();
+  if (oct_pq_conn.get_rep ()->octave_pq_refresh_types ())
+    error ("%s failed", fname.c_str ());
 
   return retval;
 }
